/*
 * "hello world" program, version 5:
 *
 * uses java.util.concurrent ExecutorService to start threads and assign 
 * them work, yet another way.
 *
 * command-line argument specifies number of threads.
 */
package csci3366.sample.hello;
import csci3366.sample.utility.Utility;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.ArrayList;

public class Hello5 {

    public static void main(String[] args) {

        /* get number of threads from command line */

        int numThreads = Utility.getIntegerArg(args, 0, 1, 
                "numThreads", "arguments:  numThreads");

        /* create executor for threads */

        ExecutorService executor = Executors.newFixedThreadPool(numThreads);

        /* 
         * create tasks that return values and send to executor, creating
         * Future objects so we can wait for tasks to finish (and get
         * their values.
         * 
         * the following line, which seems plausible, does not compile,
         * for reasons beyond the scope of this course! 
         *
         * Future<Integer>[] results = new Future<Integer>[numThreads]; 
         */
        ArrayList<Future<Integer>> results = new ArrayList<Future<Integer>>();

        for (int i = 0; i < numThreads; ++i) {
            results.add(executor.submit(new Inner(i)));
        }

        /* shut down executor and wait for threads to finish */

        executor.shutdown();
        for (Future<Integer> r : results) {
            try {
                System.out.println("thread "+r.get()+" finished");
            }
            /* must catch -- "checked exception" */
            catch (ExecutionException e) {
                System.err.println("should not happen");
            }
            /* must catch -- "checked exception" */
            catch (InterruptedException e) {
                System.err.println("should not happen");
            }
        }

        System.out.println("threads all done");

    }

    /* 
     * inner class containing code for each thread to execute 
     * by using Callable we allow threads to return a value 
     */

    private static class Inner implements Callable<Integer> {

        private int myID;

        public Inner(int myID_) {
            myID = myID_;
        }

        public Integer call() {
            System.out.println("hello from " + myID);
            return myID; /* uses autoboxing to create Integer */
        }

    }
}
